@@ -49,11 +49,64 @@
 
 #include "htp.h"
 
+/*
+ * SYNOPSIS:
+ *   static time_t get_gmtoffset(void);
+ *
+ * ARGUMENTS:
+ *   (none)
+ *
+ * RETURN VALUE:
+ *   This function return a time_t value that can be added to a localtime
+ *   time_t value to produce a GMT time_t value.
+ *
+ *   -1 is returned on error.
+ *
+ * NOTES:
+ *   Calculate the difference between localtime and GMT/UTC.
+ *
+ *   This function does not recalculate the difference everytime it is called
+ *   so it's possible that it will return the wrong value if your GMT offset
+ *   changes while the process is loaded.
+ *
+ */
+static time_t get_gmtoffset(void) {
+	static time_t retval = -1;
+
+	/* Return saved value. */
+	if (retval != -1) {
+		return(retval);
+	}
+
+	time(&retval);
+
+	retval -= mktime(gmtime(&retval));
+
+	return(retval);
+}
+
+/*
+ * SYNOPSIS:
+ *   static time_t mktime_from_rfc2616(
+ *                                     const char *date
+ *                                    );
+ *
+ * ARGUMENTS:
+ *   const char *date        Date in RFC2616 compliant format
+ *
+ * RETURN VALUE:
+ *   This function returns the time_t value for the date string passed
+ *   (which should be at GMT/UTC).
+ *
+ * NOTES:
+ *
+ */
 static time_t mktime_from_rfc2616(const char *date) {
+	char *monthinfo[12] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
 	struct tm timeinfo;
 	time_t currtime = -1;
-	char *monthinfo[12] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
 	unsigned int i;
+	time_t gmtoffset;
 
 	/* Require a valid parameter. */
 	if (date == NULL) {
@@ -87,12 +140,36 @@ static time_t mktime_from_rfc2616(const char *date) {
 		return(-1);
 	}
 
+	/* Calculate GMT offset. */
+	gmtoffset = get_gmtoffset();
+
 	/* Create GMT value */
-	currtime = mktime(&timeinfo);
+	currtime = mktime(&timeinfo) + gmtoffset;
 
 	return(currtime);
 }
 
+/*
+ * SYNOPSIS:
+ *   double set_clock(
+ *                    time_t timeval,
+ *                    int allow_adj
+ *                   );
+ *
+ * ARGUMENTS:
+ *   time_t timeval          Time value to set clock to. (GMT/UTC)
+ *   int allow_adj           Set to 1 if you want to allow time to be "Slew"'d
+ *                           if the OS supports this.
+ *
+ * RETURN VALUE:
+ *   This function returns the number of seconds the clock was adjusted.
+ *   0.0 is returned on failure to adjust the clock.
+ *
+ * NOTES:
+ *   0.0 is returned if the clock cannot be set, this is the same return
+ *   value as if no adjustment was needed.
+ *
+ */
 double set_clock(time_t timeval, int allow_adj) {
 	struct timeval tvinfo;
 	struct timeval tvdelta;
@@ -107,69 +184,66 @@ double set_clock(time_t timeval, int allow_adj) {
 #ifdef HAVE_ADJTIME
 	if (allow_adj) {
 		tvdelta.tv_sec = timeval - tvdelta.tv_sec;
-		tvdelta.tv_usec = 500000 - tvdelta.tv_usec; /* Assume 0.5s of error. */
-		if (tvdelta.tv_usec < 0 && tvdelta.tv_sec > 0) {
-			/* + -   ->   + + */
-			tvdelta.tv_sec--;
-			tvdelta.tv_usec += 1000000;
-		} else if (tvdelta.tv_sec < 0 && tvdelta.tv_usec > 0) {
-			/* - +   ->   - - */
-			tvdelta.tv_sec++;
-			tvdelta.tv_usec -= 1000000;
-		}
+		tvdelta.tv_usec = 0;
 	}
 
 	if ((tvdelta.tv_sec < 60 && tvdelta.tv_sec > -60) && allow_adj) {
 		chkret = adjtime(&tvdelta, NULL);
-		if (chkret >= 0) {
 #ifdef HAVE_SYSLOG
-			syslog(LOG_NOTICE, "Adjusting clock by %f seconds (skew).", (double) tvdelta.tv_sec + (((double) tvdelta.tv_usec) / 1000000.0));
-#endif
+		if (chkret >= 0) {
+			syslog(LOG_NOTICE, "Adjusting clock by %f seconds (slew).", (double) tvdelta.tv_sec + (((double) tvdelta.tv_usec) / 1000000.0));
 		}
-	} else {
-		chkret = -1;
+#endif
 	}
+#endif
 
 	if (chkret < 0) {
-#endif
 		tvinfo.tv_sec = timeval;
-		tvinfo.tv_usec = 500000; /* Estimate atleast 0.5s of error. */
+		tvinfo.tv_usec = 0;
 
 		chkret = settimeofday(&tvinfo, NULL);
-		if (chkret >= 0) {
 #ifdef HAVE_SYSLOG
-			syslog(LOG_NOTICE, "Adjusting clock by %f seconds (set).", (double) (tvinfo.tv_sec - currtime) + (((double) tvinfo.tv_usec) / 1000000.0));
-#endif
+		if (chkret >= 0) {
+			syslog(LOG_NOTICE, "Adjusting clock by %f seconds (step).", (double) (tvinfo.tv_sec - currtime) + (((double) tvinfo.tv_usec) / 1000000.0));
 		}
-#ifdef HAVE_ADJTIME
-	}
 #endif
+	}
 
 	if (chkret < 0) {
 		retval = 0.0;
 	} else {
-		retval = difftime(currtime, timeval);
+		retval = currtime - timeval;
 	}
 
 	return(retval);
 }
 
-/* Calculate the difference between localtime and GMT/UT/UTC. */
-static time_t get_gmtoffset(void) {
-	static time_t retval = -1;
-
-	/* Return saved value. */
-	if (retval != -1) {
-		return(retval);
-	}
-
-	time(&retval);
-
-	retval -= mktime(gmtime(&retval));
-
-	return(retval);
-}
-
+/*
+ * SYNOPSIS:
+ *   static time_t get_server_time(
+ *                                 const char *host,
+ *                                 unsigned int port,
+ *                                 const char *proxyhost,
+ *                                 unsigned int proxyport
+ *                                );
+ *
+ * ARGUMENTS:
+ *   const char *host        Destination host
+ *   unsigned int port       Destination port
+ *   const char *proxyhost   Proxy to send request through
+ *   unsigned int proxyport  Proxy port to connect
+ *
+ * RETURN VALUE:
+ *   This function returns the GMT/UTC time_t value that is specified
+ *   in the remote hosts HTTP/1.1 header reply.  (time_t) -1 is returned
+ *   on error.
+ *
+ * NOTES:
+ *   If `proxyhost' is not NULL all connections are made through `proxyhost'
+ *   on the `proxyport' port.  If `proxyhost' is NULL, connections are made
+ *   directly (and proxyport is ignored.)
+ *
+ */
 static time_t get_server_time(const char *host, unsigned int port, const char *proxyhost, unsigned int proxyport) {
 	unsigned int         server_s;		/* Server socket descriptor */
 	struct sockaddr_in   server_addr;	/* Server Internet address */
@@ -241,6 +315,21 @@ static time_t get_server_time(const char *host, unsigned int port, const char *p
 	return(retval);
 }
 
+/*
+ * SYNOPSIS:
+ *   static int htp_net_init(void);
+ *
+ * ARGUMENTS:
+ *   (none)
+ *
+ * RETURN VALUE:
+ *   0 is returned on success (network is ready to go), otherwise -1 is
+ *   returned (network access will fail.)
+ *
+ * NOTES:
+ *   This function is largely needed for Windows only.
+ *
+ */
 static int htp_net_init(void) {
 #ifdef _USE_WIN32_  
 	WSADATA wsaData;
@@ -257,6 +346,21 @@ static int htp_net_init(void) {
 	return(0);
 }
 
+/*
+ * SYNOPSIS:
+ *   int htp_init(void);
+ *
+ * ARGUMENTS:
+ *   (none)
+ *
+ * RETURN VALUE:
+ *   0 is returned on success (htp routines are usable), otherwise -1 is
+ *   returned (htp routines will fail.)
+ *
+ * NOTES:
+ *   This must be among the first things called from your `main' program.
+ *
+ */
 int htp_init(void) {
 	if (htp_net_init() < 0) {
 		fprintf(stderr, "Error: Couldn't initialze network routines.\n");
@@ -266,6 +370,26 @@ int htp_init(void) {
 	return(0);
 }
 
+/*
+ * SYNOPSIS:
+ *   int compare_time(
+ *                    const void *a,
+ *                    const void *b
+ *                   );
+ *
+ * ARGUMENTS:
+ *   const void *a           A pointer to a `time_t' value.
+ *   const void *b           Another pointer to a `time_t' value.
+ *
+ * RETURN VALUE:
+ *   This function returns -1, 0, or 1 if the `time_t' value pointed to by a
+ *   is less than, equal to, or greater than (respectively) the `time_t'
+ *   value pointed to be b.
+ *
+ * NOTES:
+ *   This function is meant to be passed as the 4th argument to qsort().
+ *
+ */
 int compare_time(const void *a, const void *b) {
 	const time_t *tmpptr = NULL;
 	time_t  timea = 0, timeb = 0;
@@ -285,6 +409,38 @@ int compare_time(const void *a, const void *b) {
 	return(0);
 }
 
+/*
+ * SYNOPSIS:
+ *   time_t htp_calctime(
+ *                       struct timeserver_st *timeservers,
+ *                       unsigned int totaltimeservers,
+ *                       const char *proxyhost,
+ *                       unsigned int proxyport
+ *                      );
+ *
+ * ARGUMENTS:
+ *   struct timeserver_st *timeservers      Pointer to the first element of
+ *                                          an array of `struct timeserver_st'
+ *                                          variables.
+ *   unsigned int totaltimeservers          Number of entries in timeservers
+ *                                          array.
+ *   const char *proxyhost                  Proxy hostname to connect through.
+ *   unsigned int proxyport                 Port to connect to proxy on.
+ *
+ * RETURN VALUE:
+ *   This function returns the best approximation of the current `time_t'
+ *   value (in GMT/UTC) that can be deduced from the timeservers specified.
+ *   -1 is returned on error.
+ *
+ * NOTES:
+ *   If more than 2 timeservers are specified the time values outside of one
+ *   standard deviation (+/- 34% about the mean) are discarded.  The average
+ *   of all the (non-excluded) values is returned.
+ *
+ *   This function accounts for the time it takes for itself to execute in
+ *   its return value.
+ *
+ */
 time_t htp_calctime(struct timeserver_st *timeservers, unsigned int totaltimeservers, const char *proxyhost, unsigned int proxyport) {
 	unsigned int timeind = 0, meantimeind = 0;
 	unsigned int totaltimes = 0, totalstddevtimes = 0;
@@ -294,13 +450,10 @@ time_t htp_calctime(struct timeserver_st *timeservers, unsigned int totaltimeser
 	time_t stddevavgtime = 0;
 	time_t tmptime = 0;
 	time_t newtimeval = 0;
-	time_t gmtoffset = 0;
 	time_t start_time = 0, offset_time = 0;
 	char *host = NULL;
 	int stddevtimes = 0;
 
-	gmtoffset = get_gmtoffset();
-
 	start_time = time(NULL);
 
 	mintime = start_time * 2;
@@ -371,5 +524,5 @@ time_t htp_calctime(struct timeserver_st *timeservers, unsigned int totaltimeser
 	/* Add back the time this process took. */
 	newtimeval += offset_time;
 
-	return(newtimeval + gmtoffset);
+	return(newtimeval);
 }
